//
//  SPK_CalligraphyStrokesRecorder.m
//  SPK-Calligraphy
//
//  Created by Toby Harris on 25/03/2009.
//  Copyright (c) 2009 *spark audiovisual. All rights reserved.
//
//  'SPK-Calligraphy' is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  'SPK-Calligraphy' is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "SPK_CalligraphyStrokesRecorder.h"

#define	kQCPlugIn_Name				@"SPK Calligraphy - Strokes Recorder"
#define	kQCPlugIn_Description		@"Use with Strokes Animator. While Recording Strokes Toggle is true, the X,Y,P inputs will be added to a line structure each frame. Each time the toggle is set to true, a new line is created. The output is a structure containing all these lines. Designed for use recording the strokes in handwritten sentances."

@implementation SPK_CalligraphyStrokesRecorder

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputP, inputY, inputX, inputSetOriginSignal, inputReset, inputPurgeLastStroke, inputStroking;
@dynamic outputStructureOfLineStructures;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	if([key isEqualToString:@"outputStructureOfLineStructures"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Lines Structure", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputStroking"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Recording Stroke Toggle", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputSetOriginSignal"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Update Origin Signal", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputReset"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Clear All Signal", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputPurgeLastStroke"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Clear Last Stroke Signal", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputX"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"X Position", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0], QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputY"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Y Position", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0], QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputP"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Pressure", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0.1], QCPortAttributeDefaultValueKey,
				nil];
	return nil;	
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (QCPlugInViewController*) createViewController
{
	/*
	Return a new QCPlugInViewController to edit the internal settings of this plug-in instance.
	You can return a subclass of QCPlugInViewController if necessary.
	*/
	
	return [[QCPlugInViewController alloc] initWithPlugIn:self viewNibName:@"Settings"];
}

@end

@implementation SPK_CalligraphyStrokesRecorder (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	linesArray = [[NSMutableArray alloc] initWithCapacity:5];
	lineStructure = nil;
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
	
	originX, originY = 0;
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	if ([self didValueForInputKeyChange:@"inputReset"] && self.inputReset) {
		[linesArray removeAllObjects];
	}
	
	if ([self didValueForInputKeyChange:@"inputPurgeLastStroke"] && self.inputPurgeLastStroke) {
		if ([linesArray count] > 0) [linesArray removeLastObject];
	}
	
	// If we're signalling to set the origin, lets set it
	if ([self didValueForInputKeyChange:@"inputSetOriginSignal"] && self.inputSetOriginSignal) {
		originX = self.inputX;
		originY = self.inputY;
	}
	
	// If we've just started a stroke, lets create the array to hold it
	if ([self didValueForInputKeyChange:@"inputStroking"] && self.inputStroking) {
		lineStructure = [NSMutableArray arrayWithCapacity:60];
		[linesArray addObject:lineStructure];
	}
		
	// If we're recording a stroke, lets add the current coords to the current line structure
	if (self.inputStroking) {
		NSArray* xypCoordinate;
		xypCoordinate = [NSArray arrayWithObjects:[NSNumber numberWithDouble:self.inputX - originX], [NSNumber numberWithDouble:self.inputY - originY], [NSNumber numberWithDouble:self.inputP], nil];
		[lineStructure addObject:xypCoordinate];
	}
	
	self.outputStructureOfLineStructures = linesArray;
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
	
	[linesArray release];
}

@end
